package animation;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.AxisAngle4f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.geometry.Sphere;

public class Projectile extends BranchGroup {
	
	private long startTime = 0;
	private TransformGroup distanceTG = null;
	private float velocity = 0.0f;
	private float radius = 0.1f;
	private Sphere projectile = null;
	private float maxDistance = 100.0f;
	
	public Projectile(float leftRightAngle, float upDownAngle, float velocity) {
		
		setCapability(BranchGroup.ALLOW_DETACH);
		
		Transform3D leftRightTransform = new Transform3D();
		leftRightTransform.setRotation(new AxisAngle4f(Constants.Y_AXIS, (float)Math.toRadians(leftRightAngle)));
		TransformGroup leftRightTG = new TransformGroup(leftRightTransform);
		addChild(leftRightTG);
		
		Transform3D upDownTransform = new Transform3D();
		upDownTransform.setRotation(new AxisAngle4f(Constants.X_AXIS, (float)Math.toRadians(upDownAngle)));
		TransformGroup upDownTG = new TransformGroup(upDownTransform);
		leftRightTG.addChild(upDownTG);
		
		distanceTG = new TransformGroup();
		distanceTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		upDownTG.addChild(distanceTG);
		
		projectile = new Sphere(radius, new Appearance());
		distanceTG.addChild(projectile);
		
		startTime = System.currentTimeMillis();
		this.velocity = velocity;
	}
	
	public void update() {
		float time = (System.currentTimeMillis() - startTime) / 1000.0f;
		float distance = time * velocity;
		if(distance > maxDistance) {
			detach();
		}
		else {
			Transform3D distanceTransform = new Transform3D();
			distanceTransform.setTranslation(new Vector3f(0.0f, distance, 0.0f));
			distanceTG.setTransform(distanceTransform);
		}
	}
	
	public Point3f getLocation() {
		Transform3D localToVworldTransform = new Transform3D();
		projectile.getLocalToVworld(localToVworldTransform);
		Vector3f translation = new Vector3f();
		localToVworldTransform.get(translation);
		return new Point3f(translation.x, translation.y, translation.z);
	}
	
	public float getRadius() {
		return radius;
	}
}
