package animation;

import java.awt.event.KeyEvent;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.AxisAngle4f;

import com.sun.j3d.utils.geometry.Cylinder;

public class ProjectilesAndCollisions extends AnimatedScene implements UpdateListener {
	
	private float upDownAngle = -90.f;
	private float leftRightAngle = 0.0f;
	
	private TransformGroup upDownTG = null;
	private TransformGroup leftRightTG = null;
	private BranchGroup projectiles = null;
	private BranchGroup targets = null;
	
	public ProjectilesAndCollisions() {
		enableAnimationBehavior(this);
		enableKeyListener();
		createContentBranch();
	}
	
	public void createContentBranch() {
		
		BranchGroup contentRoot = new BranchGroup();
		
		leftRightTG = new TransformGroup();
		leftRightTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		contentRoot.addChild(leftRightTG);
		
		Transform3D upDownTransform = new Transform3D();
		upDownTransform.setRotation(new AxisAngle4f(Constants.X_AXIS, (float)Math.toRadians(upDownAngle)));
		upDownTG = new TransformGroup(upDownTransform);
		upDownTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		leftRightTG.addChild(upDownTG);
		
		Cylinder cylinder = new Cylinder(0.1f, 0.5f, new Appearance());
		upDownTG.addChild(cylinder);
		
		projectiles = new BranchGroup();
		projectiles.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
		projectiles.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		contentRoot.addChild(projectiles);
		
		targets = new BranchGroup();
		targets.addChild(new Target(-2.0f, 2.0f, -10.0f, 1.0f));
		targets.addChild(new Target(2.0f, 2.0f, -15.0f, 1.0f));
		targets.addChild(new Target(-2.0f, -2.0f, -7.0f, 1.0f));
		targets.addChild(new Target(2.0f, -2.0f, -12.0f, 1.0f));
		targets.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
		contentRoot.addChild(targets);
		
		getSimpleUniverse().addBranchGraph(contentRoot);
	}
	
	long previousFrameTime = -1;
	
	public void updateSceneGraph() {
		
		long currentFrameTime = System.currentTimeMillis();
		if(previousFrameTime != -1) {
			int interFrameTime = (int)(currentFrameTime - previousFrameTime);
			int framesPerSecond = 1000 / interFrameTime;
			System.out.println(framesPerSecond+" FPS");
		}
		previousFrameTime = currentFrameTime;
		
		
		for(int p=0; p<projectiles.numChildren(); p++) {
			Projectile currentProjectile = (Projectile)projectiles.getChild(p);
			currentProjectile.update();
			
			for(int t=0; t<targets.numChildren(); t++) {
				Target currentTarget = (Target)targets.getChild(t);
				
				if(currentProjectile.getLocation().distance(currentTarget.getLocation()) <
						currentProjectile.getRadius()+currentTarget.getRadius()) {
					currentTarget.detach();
					currentProjectile.detach();
					 break;
				}
			}
		}
	}
	
	public void keyReleased(KeyEvent keyEvent) {

		int keyCode = keyEvent.getKeyCode();
		if(keyCode == KeyEvent.VK_UP) {
			upDownAngle -= 5.0f;
		}
		else if(keyCode == KeyEvent.VK_DOWN) {
			upDownAngle += 5.0f;
		}
		else if(keyCode == KeyEvent.VK_LEFT) {
			leftRightAngle += 5.0f;
		}
		else if(keyCode == KeyEvent.VK_RIGHT) {
			leftRightAngle -= 5.0f;
		}
		else if(keyCode == KeyEvent.VK_SPACE) {
			Projectile projectile = new Projectile(leftRightAngle, upDownAngle, 15.0f);
			projectiles.addChild(projectile);
		}
		
		Transform3D leftRightTransform = new Transform3D();
		leftRightTransform.setRotation(new AxisAngle4f(Constants.Y_AXIS, (float)Math.toRadians(leftRightAngle)));
		leftRightTG.setTransform(leftRightTransform);
		
		Transform3D upDownTransform = new Transform3D();
		upDownTransform.setRotation(new AxisAngle4f(Constants.X_AXIS, (float)Math.toRadians(upDownAngle)));
		upDownTG.setTransform(upDownTransform);
	}
	
	public static void main(String args[]) {
		new ProjectilesAndCollisions();
	}

}
